@@ -30,13 +30,11 @@
 #include <libgnomevfs/gnome-vfs-uri.h>
 #include <libgnomevfs/gnome-vfs-ops.h>
 #include <gst/gst.h>
+#include <gst/tag/tag.h>
 #include <string.h>
 #include <profiles/gnome-media-profiles.h>
 #include <gtk/gtk.h>
 
-#ifdef HAVE_GSTREAMER_0_10
-#include <gst/tag/tag.h>
-#endif
 
 #include "rhythmdb.h"
 #include "eel-gconf-extensions.h"
@@ -44,13 +42,7 @@
 #include "rb-encoder.h"
 #include "rb-encoder-gst.h"
 #include "rb-debug.h"
-
-#ifdef HAVE_GSTREAMER_0_8
-#define GstStateChangeReturn GstElementStateReturn
-#define GstState GstElementState
-#define GST_STATE_CHANGE_FAILURE GST_STATE_FAILURE
-#define gst_caps_unref gst_caps_free
-#endif
+#include "rb-util.h"
 
 static void rb_encoder_gst_class_init (RBEncoderGstClass *klass);
 static void rb_encoder_gst_init       (RBEncoderGst *encoder);
@@ -80,18 +72,50 @@ G_DEFINE_TYPE_WITH_CODE(RBEncoderGst, rb_encoder_gst, G_TYPE_OBJECT,
 static gboolean rb_encoder_gst_encode (RBEncoder *encoder,
 				       RhythmDBEntry *entry,
 				       const char *dest,
-				       const char *mime_type);
+				       GList *mime_types);
 static void rb_encoder_gst_cancel (RBEncoder *encoder);
+static gboolean rb_encoder_gst_get_preferred_mimetype (RBEncoder *encoder,
+						       GList *mime_types,
+						       char **mime,
+						       char **extension);
 static void rb_encoder_gst_emit_completed (RBEncoderGst *encoder);
 
+
 static void
 rb_encoder_gst_class_init (RBEncoderGstClass *klass)
 {
         GObjectClass *object_class = (GObjectClass *) klass;
+	GstCaps *caps;
 
         object_class->finalize = rb_encoder_gst_finalize;
 
         g_type_class_add_private (klass, sizeof (RBEncoderGstPrivate));
+
+	/* create the mimetype -> GstCaps lookup table
+	 *
+	 * The strings are static data for now, but if we allow dynamic changing
+	 * we need to change this to use g_strdup/g_free
+	 */
+	klass->mime_caps_table = g_hash_table_new_full (g_str_hash, g_str_equal,
+							NULL, (GDestroyNotify)gst_caps_unref);
+
+	/* AAC */
+	caps = gst_caps_new_simple ("audio/mpeg",
+				    "mpegversion", G_TYPE_INT, 4,
+				    NULL);
+	g_hash_table_insert (klass->mime_caps_table, "audio/aac", caps);
+
+	/* MP3 */
+	caps = gst_caps_new_simple ("audio/mpeg",
+				    "mpegversion", G_TYPE_INT, 1,
+				    "layer", G_TYPE_INT, 3,
+				    NULL);
+	g_hash_table_insert (klass->mime_caps_table, "audio/mpeg", caps);
+
+	/* hack for HAL's application/ogg reporting, assume it's audio/vorbis */
+	caps = gst_caps_new_simple ("audio/x-vorbis",
+				    NULL);
+	g_hash_table_insert (klass->mime_caps_table, "application/ogg", caps);
 }
 
 static void
@@ -105,6 +129,7 @@ rb_encoder_init (RBEncoderIface *iface)
 {
 	iface->encode = rb_encoder_gst_encode;
 	iface->cancel = rb_encoder_gst_cancel;
+	iface->get_preferred_mimetype = rb_encoder_gst_get_preferred_mimetype;
 }
 
 static void
@@ -166,7 +191,6 @@ rb_encoder_gst_emit_completed (RBEncoderGst *encoder)
 	_rb_encoder_emit_completed (RB_ENCODER (encoder));
 }
 
-#ifdef HAVE_GSTREAMER_0_10
 static gboolean
 bus_watch_cb (GstBus *bus, GstMessage *message, gpointer data)
 {
@@ -214,36 +238,6 @@ bus_watch_cb (GstBus *bus, GstMessage *message, gpointer data)
 	g_object_unref (G_OBJECT (encoder));
 	return TRUE;
 }
-#endif
-
-#ifdef HAVE_GSTREAMER_0_8
-static void
-gst_eos_cb (GstElement *element, RBEncoderGst *encoder)
-{
-	rb_debug ("received EOS");
-
-	gst_element_set_state (encoder->priv->pipeline, GST_STATE_NULL);
-
-	rb_encoder_gst_emit_completed (encoder);
-
-	g_object_unref (encoder->priv->pipeline);
-	encoder->priv->pipeline = NULL;
-}
-
-static void
-gst_error_cb (GstElement *element,
-			  GstElement *source,
-			  GError *error,
-			  gchar *debug,
-			  RBEncoderGst *encoder)
-{
-	rb_encoder_gst_emit_error (encoder, error);
-	rb_debug ("received error %s", debug);
-
-	rb_encoder_cancel (RB_ENCODER (encoder));
-}
-
-#endif
 
 static gboolean
 progress_timeout_cb (RBEncoderGst *encoder)
@@ -257,7 +251,6 @@ progress_timeout_cb (RBEncoderGst *encoder)
 
 	format = encoder->priv->position_format;
 
-#ifdef HAVE_GSTREAMER_0_10
 	gst_element_get_state (encoder->priv->pipeline, &state, NULL, GST_CLOCK_TIME_NONE);
 	if (state != GST_STATE_PLAYING)
 		return FALSE;
@@ -266,16 +259,6 @@ progress_timeout_cb (RBEncoderGst *encoder)
 		g_warning ("Could not get current track position");
 		return TRUE;
 	}
-#elif HAVE_GSTREAMER_0_8
-	state = gst_element_get_state (encoder->priv->pipeline);
-	if (state != GST_STATE_PLAYING)
-		return FALSE;
-
-	if (!gst_element_query (encoder->priv->pipeline, GST_QUERY_POSITION, &format, &position)) {
-		g_warning ("Could not get current track position");
-		return TRUE;
-	}
-#endif
 
 	if (format == GST_FORMAT_TIME) {
 		gint secs;
@@ -284,7 +267,7 @@ progress_timeout_cb (RBEncoderGst *encoder)
 		rb_debug ("encoding progress at %d out of %" G_GINT64_FORMAT,
 			  secs,
 			  encoder->priv->total_length);
-		_rb_encoder_emit_progress (RB_ENCODER (encoder), 
+		_rb_encoder_emit_progress (RB_ENCODER (encoder),
 					   ((double)secs) / encoder->priv->total_length);
 	} else {
 		rb_debug ("encoding progress at %" G_GINT64_FORMAT " out of %" G_GINT64_FORMAT,
@@ -301,7 +284,6 @@ static gboolean
 start_pipeline (RBEncoderGst *encoder, GError **error)
 {
 	GstStateChangeReturn result;
-#ifdef HAVE_GSTREAMER_0_10
 	GstBus *bus;
 
 	g_return_val_if_fail (encoder->priv->pipeline != NULL, FALSE);
@@ -310,18 +292,6 @@ start_pipeline (RBEncoderGst *encoder, GError **error)
 	gst_bus_add_watch (bus, bus_watch_cb, encoder);
 
 	result = gst_element_set_state (encoder->priv->pipeline, GST_STATE_PLAYING);
-#elif HAVE_GSTREAMER_0_8
-
-	g_return_val_if_fail (encoder->priv->pipeline != NULL, FALSE);
-
-	g_signal_connect_object (G_OBJECT (encoder->priv->pipeline),
-				 "error", G_CALLBACK (gst_error_cb),
-				 encoder, 0);
-	g_signal_connect_object (G_OBJECT (encoder->priv->pipeline),
-				 "eos", G_CALLBACK (gst_eos_cb),
-				 encoder, 0);
-	result = gst_element_set_state (encoder->priv->pipeline, GST_STATE_PLAYING);
-#endif
 
 	if (result != GST_STATE_CHANGE_FAILURE) {
 		/* start reporting progress */
@@ -336,37 +306,7 @@ start_pipeline (RBEncoderGst *encoder, GError **error)
 	return (result != GST_STATE_CHANGE_FAILURE);
 }
 
-#ifdef HAVE_GSTREAMER_0_8
-/* this is basically what the function in 0.10 does */
-static GstPad*
-rb_gst_bin_find_unconnected_pad (GstBin *bin, GstPadDirection dir)
-{
-	const GList *elements, *el;
-
-	elements = gst_bin_get_list (GST_BIN (bin));
-	for (el = elements; el != NULL; el = el->next) {
-		GstElement *element = GST_ELEMENT (el->data);
-		const GList *pads, *pl;
-
-		pads = gst_element_get_pad_list (element);
-
-		for (pl = pads; pl != NULL; pl = pl->next) {
-			GstPad *pad = GST_PAD (pl->data);
-
-			if (!GST_PAD_IS_LINKED (pad) && GST_PAD_DIRECTION (pad) == dir)
-				return pad;
-		}
-	}
-
-	return NULL;
-}
-
-#define gst_bin_find_unconnected_pad rb_gst_bin_find_unconnected_pad
-
-const char *GST_ENCODING_PROFILE = "audioscale ! audioconvert ! %s";
-#elif HAVE_GSTREAMER_0_10
-const char *GST_ENCODING_PROFILE = "audioresample ! audioconvert ! %s";
-#endif
+static const char *GST_ENCODING_PROFILE = "audioresample ! audioconvert ! %s";
 
 static GstElement*
 add_encoding_pipeline (RBEncoderGst *encoder,
@@ -455,9 +395,28 @@ add_tags_from_entry (RBEncoderGst *encoder,
 				  GST_TAG_MUSICBRAINZ_TRACKID, rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_MUSICBRAINZ_TRACKID),
 				  NULL);
 	}
+	if (rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_MUSICBRAINZ_ARTISTID)) {
+		gst_tag_list_add (tags, GST_TAG_MERGE_APPEND,
+				  GST_TAG_MUSICBRAINZ_ARTISTID, rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_MUSICBRAINZ_ARTISTID),
+				  NULL);
+	}
+	if (rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_MUSICBRAINZ_ALBUMID)) {
+		gst_tag_list_add (tags, GST_TAG_MERGE_APPEND,
+				  GST_TAG_MUSICBRAINZ_ALBUMID, rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_MUSICBRAINZ_ALBUMID),
+				  NULL);
+	}
+	if (rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_MUSICBRAINZ_ALBUMARTISTID)) {
+		gst_tag_list_add (tags, GST_TAG_MERGE_APPEND,
+				  GST_TAG_MUSICBRAINZ_ALBUMARTISTID, rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_MUSICBRAINZ_ALBUMARTISTID),
+				  NULL);
+	}
+	if (rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_MUSICBRAINZ_ARTISTSORTNAME)) {
+		gst_tag_list_add (tags, GST_TAG_MERGE_APPEND,
+				  GST_TAG_MUSICBRAINZ_SORTNAME, rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_MUSICBRAINZ_ARTISTSORTNAME),
+				  NULL);
+	}
 #endif
 
-#ifdef HAVE_GSTREAMER_0_10
 	{
 		GstIterator *iter;
 		gboolean done;
@@ -492,15 +451,6 @@ add_tags_from_entry (RBEncoderGst *encoder,
 		}
 		gst_iterator_free (iter);
 	}
-#elif HAVE_GSTREAMER_0_8
-	{
-		GstElement *tagger;
-
-		tagger = gst_bin_get_by_interface (GST_BIN (encoder->priv->pipeline), GST_TYPE_TAG_SETTER);
-		if (tagger)
-			gst_tag_setter_merge (GST_TAG_SETTER (tagger), tags, GST_TAG_MERGE_REPLACE_ALL);
-	}
-#endif
 
 	gst_tag_list_free (tags);
 	return result;
@@ -617,21 +567,59 @@ attach_output_pipeline (RBEncoderGst *encoder,
 }
 
 static gboolean
-encoder_match_mime (GstElement *encoder, const gchar *mime_type)
+encoder_match_mime (RBEncoderGst *rbencoder, GstElement *encoder, const gchar *mime_type)
 {
 	GstPad *srcpad;
-	GstCaps *caps;
-	GstStructure *structure;
-	const gchar *pad_mime;
-	gboolean match;
+	GstCaps *element_caps = NULL;
+	GstCaps *desired_caps = NULL;
+	GstCaps *intersect_caps = NULL;
+	gboolean match = FALSE;
+	char *tmp;
 
 	srcpad = gst_element_get_pad (encoder, "src");
-	caps = gst_pad_get_caps (srcpad);
-	structure = gst_caps_get_structure (caps, 0);
-	pad_mime = gst_structure_get_name (structure);
-	match = strcmp (mime_type, pad_mime) == 0;
-	gst_caps_unref (caps);
-	gst_object_unref (GST_OBJECT (srcpad));
+	element_caps = gst_pad_get_caps (srcpad);
+
+	if (element_caps == NULL) {
+		g_warning ("couldn't create any element caps");
+		goto end;
+	}
+
+	desired_caps = g_hash_table_lookup (RB_ENCODER_GST_GET_CLASS (rbencoder)->mime_caps_table, mime_type);
+	if (desired_caps != NULL) {
+		gst_caps_ref (desired_caps);
+	} else {
+		desired_caps = gst_caps_new_simple (mime_type, NULL);
+	}
+
+	if (desired_caps == NULL) {
+		g_warning ("couldn't create any desired caps for mimetype: %s", mime_type);
+		goto end;
+	}
+
+	intersect_caps = gst_caps_intersect (desired_caps, element_caps);
+	match = !gst_caps_is_empty (intersect_caps);
+
+	tmp = gst_caps_to_string (desired_caps);
+	rb_debug ("desired caps are: %s", tmp);
+	g_free (tmp);
+
+	tmp = gst_caps_to_string (element_caps);
+	rb_debug ("element caps are: %s", tmp);
+	g_free (tmp);
+
+	tmp = gst_caps_to_string (intersect_caps);
+	rb_debug ("intersect caps are: %s", tmp);
+	g_free (tmp);
+
+end:
+	if (intersect_caps != NULL)
+		gst_caps_unref (intersect_caps);
+	if (desired_caps != NULL)
+		gst_caps_unref (desired_caps);
+	if (element_caps != NULL)
+		gst_caps_unref (element_caps);
+	if (srcpad != NULL)
+		gst_object_unref (GST_OBJECT (srcpad));
 
 	return match;
 }
@@ -639,7 +627,6 @@ encoder_match_mime (GstElement *encoder, const gchar *mime_type)
 static GstElement *
 profile_bin_find_encoder (GstBin *profile_bin)
 {
-#ifdef HAVE_GSTREAMER_0_10
 	GstElementFactory *factory;
 	GstElement *encoder = NULL;
 	GstIterator *iter;
@@ -652,7 +639,7 @@ profile_bin_find_encoder (GstBin *profile_bin)
 		switch (gst_iterator_next (iter, &data)) {
 			case GST_ITERATOR_OK:
 				factory = gst_element_get_factory (GST_ELEMENT (data));
-				if (strcmp (factory->details.klass,
+				if (rb_safe_strcmp (factory->details.klass,
 						"Codec/Encoder/Audio") == 0) {
 					encoder = GST_ELEMENT (data);
 					done = TRUE;
@@ -674,13 +661,10 @@ profile_bin_find_encoder (GstBin *profile_bin)
 	gst_iterator_free (iter);
 
 	return encoder;
-#else
-	return NULL;
-#endif
 }
 
 static GMAudioProfile*
-get_profile_from_mime_type (const char *mime_type)
+get_profile_from_mime_type (RBEncoderGst *rbencoder, const char *mime_type)
 {
 	GList *profiles, *walk;
 	gchar *pipeline_description;
@@ -690,6 +674,8 @@ get_profile_from_mime_type (const char *mime_type)
 	GMAudioProfile *matching_profile = NULL;
 	GError *error = NULL;
 
+	rb_debug ("Looking up profile for mimetype '%s'", mime_type);
+
 	profiles = gm_audio_profile_get_active_list ();
 	for (walk = profiles; walk; walk = g_list_next (walk)) {
 		profile = (GMAudioProfile *) walk->data;
@@ -710,7 +696,7 @@ get_profile_from_mime_type (const char *mime_type)
 			continue;
 		}
 
-		if (encoder_match_mime (encoder, mime_type)) {
+		if (encoder_match_mime (rbencoder, encoder, mime_type)) {
 			matching_profile = profile;
 			gst_object_unref (GST_OBJECT (encoder));
 			gst_object_unref (GST_OBJECT (pipeline));
@@ -721,9 +707,29 @@ get_profile_from_mime_type (const char *mime_type)
 		gst_object_unref (GST_OBJECT (pipeline));
 	}
 
+	if (matching_profile)
+		g_object_ref (matching_profile);
+	g_list_free (profiles);
+
 	return matching_profile;
 }
 
+static GMAudioProfile*
+get_profile_from_mime_types (RBEncoderGst *rbencoder, GList *mime_types)
+{
+	GList *l;
+
+	for (l = mime_types; l != NULL; l = g_list_next (l)) {
+		GMAudioProfile *profile;
+
+		profile = get_profile_from_mime_type (rbencoder, (const char *)l->data);
+		if (profile)
+			return profile;
+	}
+
+	return NULL;
+}
+
 static GstElement *
 create_pipeline_and_source (RBEncoderGst *encoder,
 			    RhythmDBEntry *entry,
@@ -845,7 +851,7 @@ static gboolean
 transcode_track (RBEncoderGst *encoder,
 	 	 RhythmDBEntry *entry,
 		 const char *dest,
-		 const char *mime_type,
+		 GList *mime_types,
 		 GError **error)
 {
 	/* src ! decodebin ! queue ! encoding_profile ! queue ! sink */
@@ -854,14 +860,14 @@ transcode_track (RBEncoderGst *encoder,
 
 	g_assert (encoder->priv->pipeline == NULL);
 
-	profile = get_profile_from_mime_type (mime_type);
+	profile = get_profile_from_mime_types (encoder, mime_types);
 	if (profile == NULL) {
 		g_set_error (error,
 			     RB_ENCODER_ERROR,
 			     RB_ENCODER_ERROR_FORMAT_UNSUPPORTED,
 			     "Unable to locate encoding profile for mime-type "
-			     "'%s'", mime_type);
-		return FALSE;
+			     /*"'%s'", mime_type*/);
+		goto error;
 	} else {
 		rb_debug ("selected profile %s",
 				gm_audio_profile_get_name (profile));
@@ -869,27 +875,32 @@ transcode_track (RBEncoderGst *encoder,
 
 	src = create_pipeline_and_source (encoder, entry, error);
 	if (src == NULL)
-		return FALSE;
+		goto error;
 
 	decoder = add_decoding_pipeline (encoder, error);
 	if (decoder == NULL)
-		return FALSE;
+		goto error;
 
 	if (gst_element_link (src, decoder) == FALSE)
-		return FALSE;
+		goto error;
 
 	end = add_encoding_pipeline (encoder, profile, error);
 	if (end == NULL)
-		return FALSE;
+		goto error;
 
 	if (!attach_output_pipeline (encoder, end, dest, error))
-		return FALSE;
+		goto error;
 	if (!add_tags_from_entry (encoder, entry, error))
-		return FALSE;
+		goto error;
 	if (!start_pipeline (encoder, error))
-		return FALSE;
+		goto error;
 
 	return TRUE;
+error:
+	if (profile)
+		g_object_unref (profile);
+
+	return FALSE;
 }
 
 static GnomeVFSResult
@@ -946,10 +957,11 @@ static gboolean
 rb_encoder_gst_encode (RBEncoder *encoder,
 		       RhythmDBEntry *entry,
 		       const char *dest,
-		       const char *mime_type)
+		       GList *mime_types)
 {
 	RBEncoderGstPrivate *priv = RB_ENCODER_GST (encoder)->priv;
 	const char *entry_mime_type;
+	gboolean copy;
 	gboolean was_raw;
 	gboolean result;
 	GError *error = NULL;
@@ -960,6 +972,17 @@ rb_encoder_gst_encode (RBEncoder *encoder,
 	entry_mime_type = rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_MIMETYPE);
 	was_raw = g_str_has_prefix (entry_mime_type, "audio/x-raw");
 
+	/* hackish mapping of gstreamer media types to mime types; this
+	 * should be easier when we do proper (deep) typefinding.
+	 */
+	if (rb_safe_strcmp (entry_mime_type, "audio/x-wav") == 0) {
+		/* if it has a bitrate, assume it's mp3-in-wav */
+		if (rhythmdb_entry_get_ulong (entry, RHYTHMDB_PROP_BITRATE) != 0)
+			entry_mime_type = "audio/mpeg";
+	} else if (rb_safe_strcmp (entry_mime_type, "application/x-id3") == 0) {
+		entry_mime_type = "audio/mpeg";
+	}
+
 	vfsresult = create_parent_dirs (dest);
 	if (vfsresult != GNOME_VFS_OK) {
 		error = g_error_new_literal (RB_ENCODER_ERROR,
@@ -972,7 +995,26 @@ rb_encoder_gst_encode (RBEncoder *encoder,
 		return FALSE;
 	}
 
-	if ((mime_type == NULL && !was_raw) || (mime_type && (strcmp (mime_type, entry_mime_type) == 0))) {
+	if (mime_types == NULL) {
+		/* don't copy raw audio */
+		copy = !was_raw;
+	} else {
+		GList *l;
+
+		/* see if it's already in any of the destination formats */
+		copy = FALSE;
+		for (l = mime_types; l != NULL; l = g_list_next (l)) {
+			rb_debug ("Comparing mimetypes '%s' '%s'", entry_mime_type, (char *)l->data);
+			if (rb_safe_strcmp (entry_mime_type, l->data) == 0) {
+				rb_debug ("Matched mimetypes '%s' '%s'", entry_mime_type, (char *)l->data);
+
+				copy = TRUE;
+				break;
+			}
+		}
+	}
+
+	if (copy) {
 		priv->total_length = rhythmdb_entry_get_uint64 (entry, RHYTHMDB_PROP_FILE_SIZE);
 		priv->position_format = GST_FORMAT_BYTES;
 
@@ -981,10 +1023,10 @@ rb_encoder_gst_encode (RBEncoder *encoder,
 		priv->total_length = rhythmdb_entry_get_ulong (entry, RHYTHMDB_PROP_DURATION);
 		priv->position_format = GST_FORMAT_TIME;
 
-		if (mime_type == NULL) {
+		if (mime_types == NULL) {
 			result = extract_track (RB_ENCODER_GST (encoder), entry, dest, &error);
 		} else {
-			result = transcode_track (RB_ENCODER_GST (encoder), entry, dest, mime_type, &error);
+			result = transcode_track (RB_ENCODER_GST (encoder), entry, dest, mime_types, &error);
 		}
 	}
 
@@ -1003,3 +1045,32 @@ rb_encoder_gst_encode (RBEncoder *encoder,
 
 	return result;
 }
+
+static gboolean
+rb_encoder_gst_get_preferred_mimetype (RBEncoder *encoder,
+				       GList *mime_types,
+				       char **mime,
+				       char **extension)
+{
+	GList *l;
+
+	g_return_val_if_fail (mime_types != NULL, FALSE);
+	g_return_val_if_fail (mime != NULL, FALSE);
+	g_return_val_if_fail (extension != NULL, FALSE);
+
+	for (l = mime_types; l != NULL; l = g_list_next (l)) {
+		GMAudioProfile *profile;
+		const char *mimetype;
+
+		mimetype = (const char *)l->data;
+		profile = get_profile_from_mime_type (RB_ENCODER_GST (encoder), mimetype);
+		if (profile) {
+			*extension = g_strdup (gm_audio_profile_get_extension (profile));
+			*mime = g_strdup (mimetype);
+			g_object_unref (profile);
+			return TRUE;
+		}
+	}
+
+	return FALSE;
+}
