@@ -31,6 +31,7 @@
 #include <libgnomevfs/gnome-vfs-ops.h>
 #include <libgnomevfs/gnome-vfs-directory.h>
 #include <unistd.h>
+#include <stdlib.h>
 
 #include "rb-file-helpers.h"
 #include "rb-debug.h"
@@ -38,7 +39,9 @@
 static GHashTable *files = NULL;
 
 static char *dot_dir = NULL;
+#ifndef HAVE_G_GET_USER_SPECIAL_DIR
 static char *music_dir = NULL;
+#endif
 
 const char *
 rb_file (const char *filename)
@@ -91,8 +94,10 @@ rb_dot_dir (void)
 	return dot_dir;
 }
 
+
+#ifndef HAVE_G_GET_USER_SPECIAL_DIR
+
 /* Copied from xdg-user-dir-lookup.c */
-#include <stdlib.h>
 
 static char *
 xdg_user_dir_lookup (const char *type)
@@ -222,6 +227,25 @@ rb_music_dir (void)
 	return (const char *) music_dir;
 }
 
+#else		/* HAVE_G_GET_USER_SPECIAL_DIR */
+
+const char *
+rb_music_dir (void)
+{
+	const char *dir;
+	dir = g_get_user_special_dir (G_USER_DIRECTORY_MUSIC);
+	if (dir == NULL) {
+		dir = getenv ("HOME");
+		if (dir == NULL) {
+			dir = "/tmp";
+		}
+	}
+	rb_debug ("user music dir: %s", dir);
+	return dir;
+}
+
+#endif
+
 void
 rb_file_helpers_init (void)
 {
@@ -235,8 +259,10 @@ void
 rb_file_helpers_shutdown (void)
 {
 	g_hash_table_destroy (files);
-	g_free (music_dir);
 	g_free (dot_dir);
+#ifndef HAVE_G_GET_USER_SPECIAL_DIR
+	g_free (music_dir);
+#endif
 }
 
 #define MAX_LINK_LEVEL 5
@@ -852,6 +878,7 @@ rb_uri_handle_recursively_cb (const gchar *rel_path,
 	char *path, *escaped_rel_path;
 	char *sep;
 	gboolean dir;
+	gboolean ret;
 
 	dir = (info->type == GNOME_VFS_FILE_TYPE_DIRECTORY);
 
@@ -870,11 +897,12 @@ rb_uri_handle_recursively_cb (const gchar *rel_path,
 	escaped_rel_path = gnome_vfs_escape_path_string (rel_path);
 	escaped_rel_path = escape_extra_gnome_vfs_chars (escaped_rel_path);
 	path = g_build_filename (data->uri, escaped_rel_path, NULL);
-	(data->func) (path, dir, data->user_data);
+	ret = (data->func) (path, dir, data->user_data);
 	g_free (escaped_rel_path);
 	g_free (path);
 
-	*recurse = !recursing_will_loop;
+	/* if recursing will not loop, and function wants to recurse, then do so */
+	*recurse = !recursing_will_loop && ret;
 	return TRUE;
 }
 
@@ -956,7 +984,7 @@ _recurse_async_data_free (RBUriHandleRecursivelyAsyncData *data)
 }
 
 /* runs in worker thread */
-static void
+static gboolean
 _recurse_async_cb (const char *uri, gboolean dir, RBUriHandleRecursivelyAsyncData *data)
 {
 	g_mutex_lock (data->results_lock);
@@ -967,6 +995,7 @@ _recurse_async_cb (const char *uri, gboolean dir, RBUriHandleRecursivelyAsyncDat
 		g_idle_add ((GSourceFunc)_recurse_async_idle_cb, data);
 
 	g_mutex_unlock (data->results_lock);
+	return TRUE;
 }
 
 static gpointer
@@ -1041,41 +1070,37 @@ rb_canonicalise_uri (const char *uri)
 {
 	char *result = NULL;
 
+	g_return_val_if_fail (uri != NULL, NULL);
+
 	if (uri[0] == '/') {
 		/* local path */
 		char *tmp;
 		result = gnome_vfs_make_path_name_canonical (uri);
-		tmp = gnome_vfs_escape_path_string (result);
+		tmp = gnome_vfs_get_uri_from_local_path (result);
 		g_free (result);
 		if (tmp == NULL)
 			return NULL;
-		tmp = escape_extra_gnome_vfs_chars (tmp);
-		result = g_strconcat ("file://", tmp, NULL);
-		g_free (tmp);
-	} else  if (g_str_has_prefix (uri, "file://")) {
-	    	/* local file, rhythmdb wants this path escaped */
-		char *tmp1, *tmp2;
-		tmp1  = gnome_vfs_unescape_string (uri + 7, NULL);  /* ignore "file://" */
-		tmp2 = gnome_vfs_escape_path_string (tmp1);
-		g_free (tmp1);
-		if (tmp2 == NULL)
-			return NULL;
-		tmp2 = escape_extra_gnome_vfs_chars (tmp2);
-		result = g_strconcat ("file://", tmp2, NULL); /* re-add scheme */
-		g_free (tmp2);
-	} else {
-		GnomeVFSURI *vfsuri = gnome_vfs_uri_new (uri);
+		result = tmp;
+	} else if (strstr (uri, "://") == NULL) {
+		/* local relative path */
+		char *curdir, *escaped, *curdir_withslash;
 
-		if (vfsuri != NULL) {
-			/* non-local uri, leave as-is */
-			gnome_vfs_uri_unref (vfsuri);
+		curdir = g_get_current_dir ();
+		escaped = gnome_vfs_escape_path_string (curdir);
+		curdir_withslash = g_strdup_printf ("file://%s%c",
+						    escaped, G_DIR_SEPARATOR);
+		g_free (escaped);
+		g_free (curdir);
+
+		escaped = gnome_vfs_escape_path_string (uri);
+		result = gnome_vfs_uri_make_full_from_relative
+			(curdir_withslash, escaped);
+		g_free (curdir_withslash);
+		g_free (escaped);
+	} else {
+		result = gnome_vfs_make_uri_canonical (uri);
+		if (result == NULL)
 			result = g_strdup (uri);
-		} else {
-			/* this may just mean that gnome-vfs doesn't recognise the
-			 * uri scheme, so return it as is */
-			rb_debug ("Error processing probable URI %s", uri);
-			result = g_strdup (uri);
-		}
 	}
 
 	return result;
@@ -1111,6 +1136,13 @@ rb_uri_append_uri (const char *uri, const char *fragment)
 		return NULL;
 	}
 
+	/* skip scheme component of the fragment */
+	if (has_valid_scheme (fragment)) {
+		while (is_valid_scheme_character (*fragment))
+			fragment++;
+	}
+	fragment++;
+
 	full_uri = gnome_vfs_uri_append_string (vfs_uri, fragment);
 	gnome_vfs_uri_unref (vfs_uri);
 	result = gnome_vfs_uri_to_string (full_uri, GNOME_VFS_URI_HIDE_NONE);
