@@ -722,7 +722,50 @@ rb_marshal_VOID__DOUBLE_LONG (GClosure     *closure,
             data2);
 }
 
-/* VOID:INT,INT,DOUBLE (./rb-marshal.list:16) */
+/* VOID:INT,INT (./rb-marshal.list:16) */
+extern void rb_marshal_VOID__INT_INT (GClosure     *closure,
+                                      GValue       *return_value,
+                                      guint         n_param_values,
+                                      const GValue *param_values,
+                                      gpointer      invocation_hint,
+                                      gpointer      marshal_data);
+void
+rb_marshal_VOID__INT_INT (GClosure     *closure,
+                          GValue       *return_value,
+                          guint         n_param_values,
+                          const GValue *param_values,
+                          gpointer      invocation_hint,
+                          gpointer      marshal_data)
+{
+  typedef void (*GMarshalFunc_VOID__INT_INT) (gpointer     data1,
+                                              gint         arg_1,
+                                              gint         arg_2,
+                                              gpointer     data2);
+  register GMarshalFunc_VOID__INT_INT callback;
+  register GCClosure *cc = (GCClosure*) closure;
+  register gpointer data1, data2;
+
+  g_return_if_fail (n_param_values == 3);
+
+  if (G_CCLOSURE_SWAP_DATA (closure))
+    {
+      data1 = closure->data;
+      data2 = g_value_peek_pointer (param_values + 0);
+    }
+  else
+    {
+      data1 = g_value_peek_pointer (param_values + 0);
+      data2 = closure->data;
+    }
+  callback = (GMarshalFunc_VOID__INT_INT) (marshal_data ? marshal_data : cc->callback);
+
+  callback (data1,
+            g_marshal_value_peek_int (param_values + 1),
+            g_marshal_value_peek_int (param_values + 2),
+            data2);
+}
+
+/* VOID:INT,INT,DOUBLE (./rb-marshal.list:17) */
 extern void rb_marshal_VOID__INT_INT_DOUBLE (GClosure     *closure,
                                              GValue       *return_value,
                                              guint         n_param_values,
@@ -767,7 +810,7 @@ rb_marshal_VOID__INT_INT_DOUBLE (GClosure     *closure,
             data2);
 }
 
-/* VOID:OBJECT,INT,INT (./rb-marshal.list:17) */
+/* VOID:OBJECT,INT,INT (./rb-marshal.list:18) */
 extern void rb_marshal_VOID__OBJECT_INT_INT (GClosure     *closure,
                                              GValue       *return_value,
                                              guint         n_param_values,
@@ -812,7 +855,7 @@ rb_marshal_VOID__OBJECT_INT_INT (GClosure     *closure,
             data2);
 }
 
-/* VOID:OBJECT,INT,INT,BOXED,UINT,UINT (./rb-marshal.list:18) */
+/* VOID:OBJECT,INT,INT,BOXED,UINT,UINT (./rb-marshal.list:19) */
 extern void rb_marshal_VOID__OBJECT_INT_INT_BOXED_UINT_UINT (GClosure     *closure,
                                                              GValue       *return_value,
                                                              guint         n_param_values,
@@ -863,7 +906,7 @@ rb_marshal_VOID__OBJECT_INT_INT_BOXED_UINT_UINT (GClosure     *closure,
             data2);
 }
 
-/* VOID:OBJECT,INT,POINTER (./rb-marshal.list:19) */
+/* VOID:OBJECT,INT,POINTER (./rb-marshal.list:20) */
 extern void rb_marshal_VOID__OBJECT_INT_POINTER (GClosure     *closure,
                                                  GValue       *return_value,
                                                  guint         n_param_values,
@@ -908,7 +951,7 @@ rb_marshal_VOID__OBJECT_INT_POINTER (GClosure     *closure,
             data2);
 }
 
-/* VOID:POINTER,INT (./rb-marshal.list:20) */
+/* VOID:POINTER,INT (./rb-marshal.list:21) */
 extern void rb_marshal_VOID__POINTER_INT (GClosure     *closure,
                                           GValue       *return_value,
                                           guint         n_param_values,
@@ -951,7 +994,7 @@ rb_marshal_VOID__POINTER_INT (GClosure     *closure,
             data2);
 }
 
-/* VOID:POINTER,INT,POINTER (./rb-marshal.list:21) */
+/* VOID:POINTER,INT,POINTER (./rb-marshal.list:22) */
 extern void rb_marshal_VOID__POINTER_INT_POINTER (GClosure     *closure,
                                                   GValue       *return_value,
                                                   guint         n_param_values,
@@ -996,7 +1039,7 @@ rb_marshal_VOID__POINTER_INT_POINTER (GClosure     *closure,
             data2);
 }
 
-/* VOID:POINTER,LONG,LONG (./rb-marshal.list:22) */
+/* VOID:POINTER,LONG,LONG (./rb-marshal.list:23) */
 extern void rb_marshal_VOID__POINTER_LONG_LONG (GClosure     *closure,
                                                 GValue       *return_value,
                                                 guint         n_param_values,
@@ -1041,7 +1084,7 @@ rb_marshal_VOID__POINTER_LONG_LONG (GClosure     *closure,
             data2);
 }
 
-/* VOID:POINTER,POINTER (./rb-marshal.list:23) */
+/* VOID:POINTER,POINTER (./rb-marshal.list:24) */
 extern void rb_marshal_VOID__POINTER_POINTER (GClosure     *closure,
                                               GValue       *return_value,
                                               guint         n_param_values,
@@ -1084,7 +1127,7 @@ rb_marshal_VOID__POINTER_POINTER (GClosure     *closure,
             data2);
 }
 
-/* VOID:POINTER,UINT (./rb-marshal.list:24) */
+/* VOID:POINTER,UINT (./rb-marshal.list:25) */
 extern void rb_marshal_VOID__POINTER_UINT (GClosure     *closure,
                                            GValue       *return_value,
                                            guint         n_param_values,
@@ -1127,7 +1170,7 @@ rb_marshal_VOID__POINTER_UINT (GClosure     *closure,
             data2);
 }
 
-/* VOID:POINTER,ULONG (./rb-marshal.list:25) */
+/* VOID:POINTER,ULONG (./rb-marshal.list:26) */
 extern void rb_marshal_VOID__POINTER_ULONG (GClosure     *closure,
                                             GValue       *return_value,
                                             guint         n_param_values,
@@ -1170,7 +1213,7 @@ rb_marshal_VOID__POINTER_ULONG (GClosure     *closure,
             data2);
 }
 
-/* VOID:STRING,DOUBLE (./rb-marshal.list:26) */
+/* VOID:STRING,DOUBLE (./rb-marshal.list:27) */
 extern void rb_marshal_VOID__STRING_DOUBLE (GClosure     *closure,
                                             GValue       *return_value,
                                             guint         n_param_values,
@@ -1213,7 +1256,7 @@ rb_marshal_VOID__STRING_DOUBLE (GClosure     *closure,
             data2);
 }
 
-/* VOID:STRING,INT (./rb-marshal.list:27) */
+/* VOID:STRING,INT (./rb-marshal.list:28) */
 extern void rb_marshal_VOID__STRING_INT (GClosure     *closure,
                                          GValue       *return_value,
                                          guint         n_param_values,
@@ -1256,7 +1299,7 @@ rb_marshal_VOID__STRING_INT (GClosure     *closure,
             data2);
 }
 
-/* VOID:STRING,POINTER (./rb-marshal.list:28) */
+/* VOID:STRING,POINTER (./rb-marshal.list:29) */
 extern void rb_marshal_VOID__STRING_POINTER (GClosure     *closure,
                                              GValue       *return_value,
                                              guint         n_param_values,
@@ -1299,7 +1342,7 @@ rb_marshal_VOID__STRING_POINTER (GClosure     *closure,
             data2);
 }
 
-/* VOID:STRING,STRING (./rb-marshal.list:29) */
+/* VOID:STRING,STRING (./rb-marshal.list:30) */
 extern void rb_marshal_VOID__STRING_STRING (GClosure     *closure,
                                             GValue       *return_value,
                                             guint         n_param_values,
@@ -1342,7 +1385,7 @@ rb_marshal_VOID__STRING_STRING (GClosure     *closure,
             data2);
 }
 
-/* VOID:STRING,STRING,OBJECT (./rb-marshal.list:30) */
+/* VOID:STRING,STRING,OBJECT (./rb-marshal.list:31) */
 extern void rb_marshal_VOID__STRING_STRING_OBJECT (GClosure     *closure,
                                                    GValue       *return_value,
                                                    guint         n_param_values,
@@ -1387,7 +1430,7 @@ rb_marshal_VOID__STRING_STRING_OBJECT (GClosure     *closure,
             data2);
 }
 
-/* VOID:STRING,STRING,STRING (./rb-marshal.list:31) */
+/* VOID:STRING,STRING,STRING (./rb-marshal.list:32) */
 extern void rb_marshal_VOID__STRING_STRING_STRING (GClosure     *closure,
                                                    GValue       *return_value,
                                                    guint         n_param_values,
@@ -1432,7 +1475,7 @@ rb_marshal_VOID__STRING_STRING_STRING (GClosure     *closure,
             data2);
 }
 
-/* VOID:STRING,STRING,STRING,UINT,BOOLEAN (./rb-marshal.list:32) */
+/* VOID:STRING,STRING,STRING,UINT,BOOLEAN (./rb-marshal.list:33) */
 extern void rb_marshal_VOID__STRING_STRING_STRING_UINT_BOOLEAN (GClosure     *closure,
                                                                 GValue       *return_value,
                                                                 guint         n_param_values,
@@ -1481,7 +1524,7 @@ rb_marshal_VOID__STRING_STRING_STRING_UINT_BOOLEAN (GClosure     *closure,
             data2);
 }
 
-/* VOID:ULONG,FLOAT (./rb-marshal.list:33) */
+/* VOID:ULONG,FLOAT (./rb-marshal.list:34) */
 extern void rb_marshal_VOID__ULONG_FLOAT (GClosure     *closure,
                                           GValue       *return_value,
                                           guint         n_param_values,
@@ -1524,7 +1567,7 @@ rb_marshal_VOID__ULONG_FLOAT (GClosure     *closure,
             data2);
 }
 
-/* VOID:OBJECT,BOOLEAN (./rb-marshal.list:34) */
+/* VOID:OBJECT,BOOLEAN (./rb-marshal.list:35) */
 extern void rb_marshal_VOID__OBJECT_BOOLEAN (GClosure     *closure,
                                              GValue       *return_value,
                                              guint         n_param_values,
@@ -1567,7 +1610,7 @@ rb_marshal_VOID__OBJECT_BOOLEAN (GClosure     *closure,
             data2);
 }
 
-/* VOID:STRING,STRING,POINTER,POINTER (./rb-marshal.list:35) */
+/* VOID:STRING,STRING,POINTER,POINTER (./rb-marshal.list:36) */
 extern void rb_marshal_VOID__STRING_STRING_POINTER_POINTER (GClosure     *closure,
                                                             GValue       *return_value,
                                                             guint         n_param_values,
