@@ -13,8 +13,8 @@
  *
  * You should have received a copy of the GNU Library General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+ * Boston, MA 02110-1301  USA.
  */
 
 #ifndef __RB_TREE_DND_H__
@@ -57,14 +57,14 @@ struct _RbTreeDragSourceIface
 	GTypeInterface g_iface;
 
 	/* VTable - not signals */
-	gboolean     (* row_draggable)        (RbTreeDragSource   *drag_source,
+	gboolean     (* rb_row_draggable)        (RbTreeDragSource   *drag_source,
 	                              GList              *path_list);
 
-	gboolean     (* drag_data_get)        (RbTreeDragSource   *drag_source,
+	gboolean     (* rb_drag_data_get)        (RbTreeDragSource   *drag_source,
                                          GList              *path_list,
                                          GtkSelectionData   *selection_data);
 
-	gboolean     (* drag_data_delete)     (RbTreeDragSource *drag_source,
+	gboolean     (* rb_drag_data_delete)     (RbTreeDragSource *drag_source,
                                          GList            *path_list);
 };
 
@@ -75,23 +75,23 @@ struct _RbTreeDragDestIface {
 
 	/* VTable - not signals */
 
-	gboolean (* drag_data_received) (RbTreeDragDest   *drag_dest,
+	gboolean (* rb_drag_data_received) (RbTreeDragDest   *drag_dest,
 					 GtkTreePath       *dest,
 					 GtkTreeViewDropPosition pos,
 					 GtkSelectionData  *selection_data);
 
-	gboolean (* row_drop_possible)  (RbTreeDragDest   *drag_dest,
+	gboolean (* rb_row_drop_possible)  (RbTreeDragDest   *drag_dest,
 					 GtkTreePath       *dest_path,
 					 GtkTreeViewDropPosition pos,
 					 GtkSelectionData  *selection_data);
 
-	gboolean (* row_drop_position) (RbTreeDragDest   *drag_dest,
+	gboolean (* rb_row_drop_position) (RbTreeDragDest   *drag_dest,
 					GtkTreePath       *dest_path,
 					GList *targets,
 					GtkTreeViewDropPosition *pos);
 
 	/* optional */
-	GdkAtom  (* get_drag_target)    (RbTreeDragDest   *drag_dest,
+	GdkAtom  (* rb_get_drag_target)    (RbTreeDragDest   *drag_dest,
 					 GtkWidget        *widget,
 					 GdkDragContext   *context,
 					 GtkTreePath      *dest_path,
