@@ -57,9 +57,12 @@ gchar** rb_string_split_words (const gchar *string);
 
 gboolean rb_string_list_equal (GList *a, GList *b);
 gboolean rb_string_list_contains (GList *list, const char *s);
-void rb_list_deep_free (GList *list);
 GList* rb_string_list_copy (GList *list);
 
+void rb_list_deep_free (GList *list);
+void rb_list_destroy_free (GList *list, GDestroyNotify destroyer);
+void rb_slist_deep_free (GSList *list);
+
 gboolean rb_str_in_strv (const char *needle, char **haystack);
 
 GList* rb_collate_hash_table_keys (GHashTable *table);
@@ -73,7 +76,7 @@ gboolean rb_signal_accumulator_object_handled (GSignalInvocationHint *hint,
 					       const GValue *handler_return,
 					       gpointer dummy);
 void rb_value_array_append_data (GValueArray *array, GType type, ...);
-void rb_value_free (GValue *val);
+void rb_value_free (GValue *val); /* g_value_unset, g_slice_free */
 
 void rb_assert_locked (GMutex *mutex);
 
