@@ -123,12 +123,14 @@ static void impl_get_status (RBSource *asource, char **text, char **progress_tex
 static gboolean impl_receive_drag (RBSource *source, GtkSelectionData *data);
 static void impl_activate (RBSource *source);
 static gboolean impl_show_popup (RBSource *source);
+static guint impl_want_uri (RBSource *source, const char *uri);
+static gboolean impl_add_uri (RBSource *source, const char *uri, const char *title, const char *genre);
 
-static void rb_lastfm_source_new_station (char *uri, char *title, RBLastfmSource *source);
+static void rb_lastfm_source_new_station (const char *uri, const char *title, RBLastfmSource *source);
 static void rb_lastfm_source_skip_track (GtkAction *action, RBLastfmSource *source);
 static void rb_lastfm_source_love_track (GtkAction *action, RBLastfmSource *source);
 static void rb_lastfm_source_ban_track (GtkAction *action, RBLastfmSource *source);
-static char *rb_lastfm_source_title_from_uri (char *uri);
+static char *rb_lastfm_source_title_from_uri (const char *uri);
 static void rb_lastfm_source_add_station_cb (GtkButton *button, gpointer *data);
 static void rb_lastfm_source_entry_added_cb (RhythmDB *db, RhythmDBEntry *entry, RBLastfmSource *source);
 
@@ -248,6 +250,8 @@ rb_lastfm_source_class_init (RBLastfmSourceClass *klass)
 	source_class->impl_receive_drag = impl_receive_drag;
 	source_class->impl_activate = impl_activate;
 	source_class->impl_show_popup = impl_show_popup;
+	source_class->impl_want_uri = impl_want_uri;
+	source_class->impl_add_uri = impl_add_uri;
 
 	g_object_class_install_property (object_class,
 					 PROP_ENTRY_TYPE,
@@ -784,7 +788,7 @@ rb_lastfm_message_cb (SoupMessage *req, gpointer user_data)
 	}
 
 	g_strstrip (body);
-	pieces = g_strsplit (body, "\n", 6);
+	pieces = g_strsplit (body, "\n", 0);
 	for (i = 0; pieces[i] != NULL; i++) {
 		gchar **values = g_strsplit (pieces[i], "=", 2);
 		if (strcmp (values[0], "session") == 0) {
@@ -907,12 +911,12 @@ rb_lastfm_proxy_config_changed_cb (RBProxyConfig *config,
 }
 
 static void
-rb_lastfm_source_new_station (char *uri, char *title, RBLastfmSource *source)
+rb_lastfm_source_new_station (const char *uri, const char *title, RBLastfmSource *source)
 {
 	RhythmDBEntry *entry;
 	GValue v = {0,};
 
-	rb_debug ("adding lastfm: %s, %s",uri, title);
+	rb_debug ("adding lastfm: %s, %s", uri, title);
 
 	entry = rhythmdb_entry_lookup_by_location (source->priv->db, uri);
 	if (entry) {
@@ -969,21 +973,47 @@ rb_lastfm_source_command (RBLastfmSource *source, const char *query_string, cons
 }
 
 static void
-rb_lastfm_source_love_track (GtkAction *action, RBLastfmSource *source)
+rb_lastfm_source_love_track (GtkAction *run_action, RBLastfmSource *source)
 {
+	GtkAction *action;
+
 	rb_lastfm_source_command (source, "command=love", _("Marking song loved..."));
+
+	/* disable love/ban */
+	action = gtk_action_group_get_action (source->priv->action_group, "LastfmLoveSong");
+	gtk_action_set_sensitive (action, FALSE);
+	action = gtk_action_group_get_action (source->priv->action_group, "LastfmBanSong");
+	gtk_action_set_sensitive (action, FALSE);
 }
 
 static void
-rb_lastfm_source_skip_track (GtkAction *action, RBLastfmSource *source)
+rb_lastfm_source_skip_track (GtkAction *run_action, RBLastfmSource *source)
 {
+	GtkAction *action;
+
 	rb_lastfm_source_command (source, "command=skip", _("Skipping song..."));
+
+	/* disable love/ban and skip */
+	action = gtk_action_group_get_action (source->priv->action_group, "LastfmSkipSong");
+	gtk_action_set_sensitive (action, FALSE);
+	action = gtk_action_group_get_action (source->priv->action_group, "LastfmLoveSong");
+	gtk_action_set_sensitive (action, FALSE);
+	action = gtk_action_group_get_action (source->priv->action_group, "LastfmBanSong");
+	gtk_action_set_sensitive (action, FALSE);
 }
 
 static void
-rb_lastfm_source_ban_track (GtkAction *action, RBLastfmSource *source)
+rb_lastfm_source_ban_track (GtkAction *run_action, RBLastfmSource *source)
 {
+	GtkAction *action;
+
 	rb_lastfm_source_command (source, "command=ban", _("Banning song..."));
+
+	/* disable love/ban */
+	action = gtk_action_group_get_action (source->priv->action_group, "LastfmLoveSong");
+	gtk_action_set_sensitive (action, FALSE);
+	action = gtk_action_group_get_action (source->priv->action_group, "LastfmBanSong");
+	gtk_action_set_sensitive (action, FALSE);
 }
 
 
@@ -1019,7 +1049,7 @@ impl_receive_drag (RBSource *asource, GtkSelectionData *selection_data)
 
 
 static char *
-rb_lastfm_source_title_from_uri (char *uri)
+rb_lastfm_source_title_from_uri (const char *uri)
 {
 	char *title = NULL;
 	char *unesc_title;
@@ -1063,7 +1093,7 @@ rb_lastfm_source_title_from_uri (char *uri)
 	}
 
 	if (title == NULL) {
-		title = g_strdup (g_strstrip (uri));
+		title = g_strstrip (g_strdup (uri));
 	}
 
 	g_strfreev (data);
@@ -1231,15 +1261,24 @@ rb_lastfm_source_new_song_cb (GObject *player_backend,
 			      gpointer data,
 			      RBLastfmSource *source)
 {
+	GtkAction *action;
 	char *uri;
-	rb_debug ("got new song");
 
+	rb_debug ("got new song");
 	uri = g_strdup_printf ("http://%s%s/np.php?session=%s&debug=0",
 			       source->priv->base_url,
 			       source->priv->base_path,
 			       source->priv->session);
 	rb_lastfm_perform (source, uri, NULL, (SoupMessageCallbackFn) rb_lastfm_source_metadata_cb);
 	g_free (uri);
+
+	/* re-enable actions */
+	action = gtk_action_group_get_action (source->priv->action_group, "LastfmSkipSong");
+	gtk_action_set_sensitive (action, TRUE);
+	action = gtk_action_group_get_action (source->priv->action_group, "LastfmLoveSong");
+	gtk_action_set_sensitive (action, TRUE);
+	action = gtk_action_group_get_action (source->priv->action_group, "LastfmBanSong");
+	gtk_action_set_sensitive (action, TRUE);
 }
 
 static gboolean
@@ -1307,3 +1346,26 @@ show_entry_popup (RBEntryView *view,
 	}
 }
 
+guint
+impl_want_uri (RBSource *source, const char *uri)
+{
+	if (g_str_has_prefix (uri, "lastfm://"))
+		return 100;
+
+	return 0;
+}
+
+static gboolean
+impl_add_uri (RBSource *source, const char *uri, const char *title, const char *genre)
+{
+	char *name;
+
+	if (strstr (uri, "lastfm://") == NULL)
+		return FALSE;
+
+	name = rb_lastfm_source_title_from_uri (uri);
+
+	rb_lastfm_source_new_station (uri, name, RB_LASTFM_SOURCE (source));
+	return TRUE;
+}
+
