@@ -83,6 +83,10 @@ struct RhythmDBEntry_ {
 	RBRefString *album;
 	RBRefString *genre;
 	RBRefString *musicbrainz_trackid;
+	RBRefString *musicbrainz_artistid;
+	RBRefString *musicbrainz_albumid;
+	RBRefString *musicbrainz_albumartistid;
+	RBRefString *musicbrainz_artistsortname;
 	gulong tracknum;
 	gulong discnum;
 	gulong duration;
@@ -147,6 +151,7 @@ struct RhythmDBPrivate
 	guint library_location_notify_id;
 	guint changed_files_id;
 	GSList *library_locations;
+	guint monitor_notify_id;
 
 	gboolean dry_run;
 	gboolean no_update;
@@ -163,8 +168,9 @@ struct RhythmDBPrivate
 
 	GCond *saving_condition;
 	GMutex *saving_mutex;
+	guint save_count;
 
-	guint event_poll_id;
+	guint event_queue_watch_id;
 	guint commit_timeout_id;
 	guint save_timeout_id;
 
@@ -199,6 +205,8 @@ typedef struct
 	RBRefString *uri;
 	RBRefString *real_uri; /* Target of a symlink, if any */
 	RhythmDBEntryType entry_type;
+	RhythmDBEntryType ignore_type;
+	RhythmDBEntryType error_type;
 
 	GError *error;
 	RhythmDB *db;
@@ -218,7 +226,7 @@ typedef struct
 } RhythmDBEvent;
 
 /* from rhythmdb.c */
-void queue_stat_uri (const char *uri, RhythmDB *db, RhythmDBEntryType type);
+void queue_stat_uri (const char *uri, RhythmDB *db, RhythmDBEntryType type, RhythmDBEntryType ignore_type, RhythmDBEntryType error_type);
 void rhythmdb_entry_set_visibility (RhythmDB *db, RhythmDBEntry *entry,
 				    gboolean visibility);
 void rhythmdb_entry_set_internal (RhythmDB *db, RhythmDBEntry *entry,
