@@ -15,7 +15,7 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA.
  *
  */
 
@@ -52,6 +52,7 @@ typedef struct
 GType             rb_shell_clipboard_get_type		(void);
 
 RBShellClipboard *rb_shell_clipboard_new		(GtkActionGroup *actiongroup,
+							 GtkUIManager *ui_mgr,
 							 RhythmDB *db);
 
 void              rb_shell_clipboard_set_source		(RBShellClipboard *shell_clipboard,
