@@ -90,10 +90,10 @@ static void impl_delete (RBSource *source);
 static void impl_search (RBSource *source, const char *text);
 static void impl_reset_filters (RBSource *source);
 static void impl_song_properties (RBSource *source);
-static gboolean impl_show_popup (RBSource *source);
 static GList *impl_get_search_actions (RBSource *source);
 static void impl_browser_toggled (RBSource *asource, gboolean disclosed);
 static void default_show_entry_popup (RBBrowserSource *source);
+static void default_pack_paned (RBBrowserSource *source, GtkWidget *paned);
 
 void rb_browser_source_browser_views_activated_cb (GtkWidget *widget,
 						 RBBrowserSource *source);
@@ -111,7 +111,6 @@ struct RBBrowserSourcePrivate
 	RhythmDB *db;
 
 	RBLibraryBrowser *browser;
-	GtkWidget *vbox;
 
 	RBEntryView *songs;
 	GtkWidget *paned;
@@ -197,10 +196,10 @@ rb_browser_source_class_init (RBBrowserSourceClass *klass)
 	source_class->impl_can_add_to_queue = (RBSourceFeatureFunc) rb_true_function;
 	source_class->impl_can_move_to_trash = (RBSourceFeatureFunc) rb_true_function;
 	source_class->impl_delete = impl_delete;
-	source_class->impl_show_popup = impl_show_popup;
 	source_class->impl_get_search_actions = impl_get_search_actions;
 	source_class->impl_browser_toggled = impl_browser_toggled;
 
+	klass->impl_pack_paned = default_pack_paned;
 	klass->impl_get_paned_key = (RBBrowserSourceStringFunc)rb_null_function;
 	klass->impl_has_drop_support = (RBBrowserSourceFeatureFunc) rb_false_function;
 	klass->impl_show_entry_popup = default_show_entry_popup;
@@ -222,10 +221,6 @@ rb_browser_source_init (RBBrowserSource *source)
 	source->priv = RB_BROWSER_SOURCE_GET_PRIVATE (source);
 
 	source->priv->search_prop = RHYTHMDB_PROP_SEARCH_MATCH;
-
-	source->priv->vbox = gtk_vbox_new (FALSE, 5);
-
-	gtk_container_add (GTK_CONTAINER (source), source->priv->vbox);
 }
 
 static void
@@ -256,6 +251,15 @@ rb_browser_source_dispose (GObject *object)
 		source->priv->cached_all_query = NULL;
 	}
 
+	if (source->priv->action_group != NULL) {
+		g_object_unref (source->priv->action_group);
+		source->priv->action_group = NULL;
+	}
+
+	eel_gconf_notification_remove (source->priv->state_browser_notify_id);
+	eel_gconf_notification_remove (source->priv->state_paned_notify_id);
+	eel_gconf_notification_remove (source->priv->state_sorting_notify_id);
+
 	G_OBJECT_CLASS (rb_browser_source_parent_class)->dispose (object);
 }
 
@@ -271,15 +275,7 @@ rb_browser_source_finalize (GObject *object)
 
 	g_return_if_fail (source->priv != NULL);
 
-	rb_debug ("finalizing browser source");
 	g_free (source->priv->sorting_key);
-	eel_gconf_notification_remove (source->priv->state_browser_notify_id);
-	eel_gconf_notification_remove (source->priv->state_paned_notify_id);
-	eel_gconf_notification_remove (source->priv->state_sorting_notify_id);
-
-	if (source->priv->action_group != NULL) {
-		g_object_unref (source->priv->action_group);
-	}
 
 	G_OBJECT_CLASS (rb_browser_source_parent_class)->finalize (object);
 }
@@ -338,6 +334,11 @@ search_action_changed (GtkRadioAction  *action,
 	RBBrowserSource *source;
 
 	g_object_get (shell, "selected-source", &source, NULL);
+	if (source == NULL || !RB_IS_BROWSER_SOURCE (source)) {
+		if (source != NULL)
+			g_object_unref (source);
+		return;
+	}
 
 	active = gtk_toggle_action_get_active (GTK_TOGGLE_ACTION (current));
 
@@ -363,10 +364,9 @@ rb_browser_source_constructor (GType type,
 	RBShell *shell;
 	GObject *shell_player;
 	char *browser_key;
+	char *paned_key;
 	RhythmDBEntryType entry_type;
 
-	klass = RB_BROWSER_SOURCE_CLASS (g_type_class_peek (RB_TYPE_BROWSER_SOURCE));
-
 	source = RB_BROWSER_SOURCE (G_OBJECT_CLASS (rb_browser_source_parent_class)
 			->constructor (type, n_construct_properties, construct_properties));
 
@@ -435,11 +435,13 @@ rb_browser_source_constructor (GType type,
 					    (GConfClientNotifyFunc) rb_browser_source_state_pref_changed,
 					    source);
 	}
-	if (rb_browser_source_get_paned_key (source)) {
+	paned_key = rb_browser_source_get_paned_key (source);
+	if (paned_key) {
 		source->priv->state_paned_notify_id =
-			eel_gconf_notification_add (rb_browser_source_get_paned_key (source),
+			eel_gconf_notification_add (paned_key,
 					    (GConfClientNotifyFunc) rb_browser_source_state_pref_changed,
 					    source);
+		g_free (paned_key);
 	}
 	browser_key = rb_source_get_browser_key (RB_SOURCE (source));
 	if (browser_key) {
@@ -477,7 +479,9 @@ rb_browser_source_constructor (GType type,
 
 	gtk_paned_pack2 (GTK_PANED (source->priv->paned), GTK_WIDGET (source->priv->songs), TRUE, FALSE);
 
-	gtk_box_pack_start_defaults (GTK_BOX (source->priv->vbox), source->priv->paned);
+	klass = RB_BROWSER_SOURCE_GET_CLASS (source);
+	klass->impl_pack_paned (source, source->priv->paned);
+
 	gtk_widget_show_all (GTK_WIDGET (source));
 
 	source->priv->cached_all_query = rhythmdb_query_model_new_empty (source->priv->db);
@@ -713,12 +717,13 @@ paned_size_allocate_cb (GtkWidget *widget,
 			GtkAllocation *allocation,
 		        RBBrowserSource *source)
 {
-	const char *key = rb_browser_source_get_paned_key (source);;
+	char *key = rb_browser_source_get_paned_key (source);;
 
 	/* save state */
 	rb_debug ("paned size allocate");
 	if (key)
 		eel_gconf_set_integer (key, gtk_paned_get_position (GTK_PANED (source->priv->paned)));
+	g_free (key);
 }
 
 static void
@@ -734,7 +739,7 @@ rb_browser_source_state_pref_changed (GConfClient *client,
 static void
 rb_browser_source_state_prefs_sync (RBBrowserSource *source)
 {
-	const char *paned_key;
+	char *paned_key;
 	char *browser_key;
 
 	rb_debug ("syncing state");
@@ -749,6 +754,8 @@ rb_browser_source_state_prefs_sync (RBBrowserSource *source)
 	} else {
 		gtk_widget_hide (GTK_WIDGET (source->priv->browser));
 	}
+
+	g_free (paned_key);
 	g_free (browser_key);
 }
 
@@ -778,14 +785,7 @@ impl_browser_toggled (RBSource *asource, gboolean disclosed)
 	}
 }
 
-static gboolean
-impl_show_popup (RBSource *source)
-{
-	_rb_source_show_popup (RB_SOURCE (source), "/BrowserSourcePopup");
-	return TRUE;
-}
-
-const char *
+char *
 rb_browser_source_get_paned_key (RBBrowserSource *source)
 {
 	RBBrowserSourceClass *klass = RB_BROWSER_SOURCE_GET_CLASS (source);
@@ -903,3 +903,14 @@ rb_browser_source_do_query (RBBrowserSource *source, gboolean subset)
 
 	rhythmdb_query_free (query);
 }
+
+static void
+default_pack_paned (RBBrowserSource *source, GtkWidget *paned)
+{
+	GtkWidget *box;
+
+	box = gtk_vbox_new (FALSE, 5);
+	gtk_box_pack_start_defaults (GTK_BOX (box), paned);
+	gtk_container_add (GTK_CONTAINER (source), box);
+}
+
