@@ -139,17 +139,24 @@ rb_removable_media_source_constructor (GType type, guint n_construct_properties,
 			->constructor (type, n_construct_properties, construct_properties);
 
 	g_object_get (source, "volume", &volume, NULL);
-	drive = gnome_vfs_volume_get_drive (volume);
-	if (drive != NULL) {
-		display_name = gnome_vfs_drive_get_display_name (drive);
-		gnome_vfs_drive_unref (drive);
+	if (volume != NULL) {
+		drive = gnome_vfs_volume_get_drive (volume);
+		if (drive != NULL) {
+			display_name = gnome_vfs_drive_get_display_name (drive);
+			gnome_vfs_drive_unref (drive);
+		} else {
+			display_name = gnome_vfs_volume_get_display_name (volume);
+		}
+		icon_name = gnome_vfs_volume_get_icon (volume);
+		g_object_unref (volume);
 	} else {
-		display_name = gnome_vfs_volume_get_display_name (volume);
+		display_name = g_strdup ("Unknown Device");
+		icon_name = g_strdup ("multimedia-player");
 	}
+
 	g_object_set (source, "name", display_name, NULL);
 	g_free (display_name);
 
-	icon_name = gnome_vfs_volume_get_icon (volume);
 	theme = gtk_icon_theme_get_default ();
 	gtk_icon_size_lookup (RB_SOURCE_ICON_SIZE, &size, NULL);
 	pixbuf = gtk_icon_theme_load_icon (theme, icon_name, size, 0, NULL);
@@ -160,8 +167,6 @@ rb_removable_media_source_constructor (GType type, guint n_construct_properties,
 		g_object_unref (pixbuf);
 	}
 
-	g_object_unref (volume);
-
 	return source;
 }
 
@@ -501,7 +506,7 @@ rb_removable_media_source_track_added (RBRemovableMediaSource *source,
 		g_object_unref (shell);
 
 		g_object_get (source, "entry-type", &entry_type, NULL);
-		rhythmdb_add_uri_with_type (db, uri, entry_type);
+		rhythmdb_add_uri_with_types (db, uri, entry_type, RHYTHMDB_ENTRY_TYPE_INVALID, RHYTHMDB_ENTRY_TYPE_INVALID);
 		g_boxed_free (RHYTHMDB_TYPE_ENTRY_TYPE, entry_type);
 
 		g_object_unref (db);
