@@ -80,6 +80,11 @@ static void rb_static_playlist_source_non_entry_dropped (GtkTreeModel *model,
 							 const char *uri,
 							 int position,
 							 RBStaticPlaylistSource *source);
+static void rb_static_playlist_source_rows_reordered (GtkTreeModel *model,
+						      GtkTreePath *path,
+						      GtkTreeIter *iter,
+						      gint *order_map,
+						      RBStaticPlaylistSource *source);
 
 static void search_action_changed (GtkRadioAction *action,
 				   GtkRadioAction *current,
@@ -242,12 +247,15 @@ rb_static_playlist_source_constructor (GType type,
 							       "StaticPlaylistActions",
 							       NULL, 0,
 							       shell);
-	gtk_action_group_add_radio_actions (priv->action_group,
-					    rb_static_playlist_source_radio_actions,
-					    G_N_ELEMENTS (rb_static_playlist_source_radio_actions),
-					    0,
-					    (GCallback)search_action_changed,
-					    shell);
+	if (gtk_action_group_get_action (priv->action_group,
+					 rb_static_playlist_source_radio_actions[0].name) == NULL) {
+		gtk_action_group_add_radio_actions (priv->action_group,
+						    rb_static_playlist_source_radio_actions,
+						    G_N_ELEMENTS (rb_static_playlist_source_radio_actions),
+						    0,
+						    (GCallback)search_action_changed,
+						    shell);
+	}
 	priv->search_prop = RHYTHMDB_PROP_SEARCH_MATCH;
 
 	g_object_unref (shell);
@@ -279,6 +287,9 @@ rb_static_playlist_source_constructor (GType type,
 	g_signal_connect_object (priv->base_model, "non-entry-dropped",
 				 G_CALLBACK (rb_static_playlist_source_non_entry_dropped),
 				 source, 0);
+	g_signal_connect_object (priv->base_model, "rows-reordered",
+				 G_CALLBACK (rb_static_playlist_source_rows_reordered),
+				 source, 0);
 
 	gtk_widget_show_all (GTK_WIDGET (source));
 
@@ -653,13 +664,14 @@ rb_static_playlist_source_add_location_internal (RBStaticPlaylistSource *source,
 	rb_playlist_source_mark_dirty (psource);
 }
 
-static void
+static gboolean
 rb_static_playlist_source_add_location_cb (const char *uri,
 					   gboolean dir,
 					   RBStaticPlaylistSource *source)
 {
 	if (!dir)
 		rb_static_playlist_source_add_location_internal (source, uri, -1);
+	return TRUE;	
 }
 
 void
@@ -778,6 +790,17 @@ rb_static_playlist_source_row_inserted (GtkTreeModel *model,
 	rhythmdb_entry_unref (entry);
 }
 
+static void
+rb_static_playlist_source_rows_reordered (GtkTreeModel *model,
+					  GtkTreePath *path,
+					  GtkTreeIter *iter,
+					  gint *order_map,
+					  RBStaticPlaylistSource *source)
+{
+	rb_playlist_source_mark_dirty (RB_PLAYLIST_SOURCE (source));
+}
+
+
 static GList *
 impl_get_search_actions (RBSource *source)
 {
