@@ -80,6 +80,7 @@ static void rb_song_info_update_bitrate (RBSongInfo *song_info);
 static void rb_song_info_update_buttons (RBSongInfo *song_info);
 static void rb_song_info_update_rating (RBSongInfo *song_info);
 static void rb_song_info_update_year (RBSongInfo *song_info);
+static void rb_song_info_update_date_added (RBSongInfo *song_info);
 static void rb_song_info_update_playback_error (RBSongInfo *song_info);
 
 static void rb_song_info_backward_clicked_cb (GtkWidget *button,
@@ -128,6 +129,7 @@ struct RBSongInfoPrivate
 	GtkWidget   *name;
 	GtkWidget   *location;
 	GtkWidget   *filesize;
+	GtkWidget   *date_added;
 	GtkWidget   *play_count;
 	GtkWidget   *last_played;
 	GtkWidget   *rating;
@@ -151,7 +153,8 @@ enum
 	PROP_0,
 	PROP_SOURCE,
 	PROP_ENTRY_VIEW,
-	PROP_CURRENT_ENTRY
+	PROP_CURRENT_ENTRY,
+	PROP_SELECTED_ENTRIES
 };
 
 static guint rb_song_info_signals[LAST_SIGNAL] = { 0 };
@@ -192,6 +195,18 @@ rb_song_info_class_init (RBSongInfoClass *klass)
 							     RHYTHMDB_TYPE_ENTRY,
 					                     G_PARAM_READABLE));
 
+	g_object_class_install_property (object_class,
+					 PROP_SELECTED_ENTRIES,
+					 g_param_spec_value_array ("selected-entries",
+								   "Selected entries",
+								   "List of selected entries, if this is a multiple-entry dialog",
+								   g_param_spec_boxed ("selected-entry",
+										       "Selected entry",
+										       "RhythmDBEntry for a selected entry",
+										       RHYTHMDB_TYPE_ENTRY,
+										       G_PARAM_READABLE),
+								   G_PARAM_READABLE));
+
 	object_class->dispose = rb_song_info_dispose;
 	object_class->finalize = rb_song_info_finalize;
 
@@ -279,6 +294,7 @@ rb_song_info_construct_single (RBSongInfo *song_info, GladeXML *xml,
 	song_info->priv->duration      = glade_xml_get_widget (xml, "song_info_duration");
 	song_info->priv->location = glade_xml_get_widget (xml, "song_info_location");
 	song_info->priv->filesize = glade_xml_get_widget (xml, "song_info_filesize");
+	song_info->priv->date_added    = glade_xml_get_widget (xml, "song_info_dateadded");
 	song_info->priv->play_count    = glade_xml_get_widget (xml, "song_info_playcount");
 	song_info->priv->last_played   = glade_xml_get_widget (xml, "song_info_lastplayed");
 	song_info->priv->name = glade_xml_get_widget (xml, "song_info_name");
@@ -288,6 +304,7 @@ rb_song_info_construct_single (RBSongInfo *song_info, GladeXML *xml,
 	rb_glade_boldify_label (xml, "name_label");
 	rb_glade_boldify_label (xml, "location_label");
 	rb_glade_boldify_label (xml, "filesize_label");
+	rb_glade_boldify_label (xml, "date_added_label");
 	rb_glade_boldify_label (xml, "last_played_label");
 	rb_glade_boldify_label (xml, "play_count_label");
 	rb_glade_boldify_label (xml, "duration_label");
@@ -636,6 +653,24 @@ rb_song_info_get_property (GObject *object,
 	case PROP_CURRENT_ENTRY:
 		g_value_set_boxed (value, song_info->priv->current_entry);
 		break;
+	case PROP_SELECTED_ENTRIES:
+		if (song_info->priv->selected_entries) {
+			GValueArray *value_array;
+			GValue entry_value = { 0, };
+			GList *entry_list;
+
+			value_array = g_value_array_new (1);
+			g_value_init (&entry_value, RHYTHMDB_TYPE_ENTRY);
+			for (entry_list = song_info->priv->selected_entries; entry_list; entry_list = entry_list->next) {
+				g_value_set_boxed (&entry_value, entry_list->data);
+				g_value_array_append (value_array, &entry_value);
+			}
+			g_value_unset (&entry_value);
+			g_value_take_boxed (value, value_array);
+		} else {
+			g_value_set_boxed (value, NULL);
+		}
+		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
 		break;
@@ -886,6 +921,7 @@ rb_song_info_populate_dialog (RBSongInfo *song_info)
 	rb_song_info_update_duration (song_info);
 	rb_song_info_update_location (song_info);
 	rb_song_info_update_filesize (song_info);
+	rb_song_info_update_date_added (song_info);
 	rb_song_info_update_play_count (song_info);
 	rb_song_info_update_last_played (song_info);
 	rb_song_info_update_bitrate (song_info);
@@ -1203,6 +1239,14 @@ rb_song_info_update_year (RBSongInfo *song_info)
 	g_free (text);
 }
 
+static void
+rb_song_info_update_date_added (RBSongInfo *song_info)
+{
+	const char *str;
+	str = rhythmdb_entry_get_string (song_info->priv->current_entry, RHYTHMDB_PROP_FIRST_SEEN_STR);
+	gtk_label_set_text (GTK_LABEL (song_info->priv->date_added), str);
+}
+
 static void
 rb_song_info_sync_entries_multiple (RBSongInfo *dialog)
 {
